package POWER::Utils::Resource;

use version; $VERSION = qv('0.0.3');    # update POD & Changes & README

use warnings;
use strict;
use Carp;

use Perl6::Export::Attrs;

use POWER::Utils;


sub MEM_used :Export(:DEFAULT) {
    return (Cat('/proc/self/status') =~ /VmRSS:\s*(\d*)/)[0];
};

sub FD_used :Export(:DEFAULT) {
    opendir my $fd, '/proc/self/fd' or croak "opendir: $!";
    return @{[ readdir $fd ]} - 2;
};

sub CPU_used :Export(:DEFAULT) {
    return (map {$_->[13]+$_->[14]} [split ' ', Cat('/proc/self/stat')])[0];
}

sub FD_list :Export(:DEFAULT) {
    croak 'Usage: FD_list( [$pid] )'
        if @_ > 1
        || (@_ == 1 && !defined $_[0])
        ;
    my $pid = @_ == 0 ? $$ : $_[0];
    my $dir = "/proc/$pid/fd";
    opendir my $dh, $dir                    or croak "opendir: $!";
    my @fd = grep {m/\A\d+\z/} readdir $dh;
    closedir $dh                            or croak "closedir: $!";
    my @list;
    for my $fd (sort {$a<=>$b} @fd) {
        my $path = "$dir/$fd";
        my $link = readlink $path;
        next if !defined $link; # skip $dh
        my $mode = (lstat $path)[2];
        $mode = sprintf "%s%s",
            ($mode & 0400 ? 'r' : ' '),
            ($mode & 0200 ? 'w' : ' ');
        push @list, [$fd, $link, $mode];
    }
    return @list;
}


1; # Magic true value required at end of module
__END__

=head1 NAME

POWER::Utils::Resource - check current resource usage


=head1 VERSION

This document describes POWER::Utils::Resource version 0.0.2


=head1 SYNOPSIS

 use POWER::Utils::Resource;

 (MEM_used() < 50000) or die "Out of memory";

 (FD_used() < 100) and open(F, "filename") or die "Out of files"; 

 printf "%3\$s %3d %s\n", @$_ for FD_list();


=head1 DESCRIPTION

Information about used resources may help with debugging or soft limiting
resources usage.


=head1 EXPORTS

=over

=item :DEFAULT

MEM_used() FD_used() CPU_used() FD_list()


=back

=head1 INTERFACE 

=over

=item B<MEM_used>()

Return: memory used by this process (in KB).


=item B<FD_used>()

Return: number of used by this process file descriptors.


=item B<CPU_used>()

Return: CPU time used by this process in seconds * 100.


=item B<FD_list>( [$pid] )

Return: Array with all fd open in process $pid (or current if executed without
params). Each element in this array will be ref to array with three elements:
[ $fd, $path, $mode ]. $mode can be: 'r ' or ' w' or 'rw' or '  '.


=back

=head1 DIAGNOSTICS

=over

=item C<< opendir: $! >>

FD_used() fail to open '/proc/self/fd' directory.

FD_list() fail to open '/proc/$pid/fd' directory.


=item C<< closedir: $! >>

FD_list() fail to close '/proc/$PID/fd' directory.


=back

=head1 CONFIGURATION AND ENVIRONMENT

POWER::Utils::Resource requires no configuration files or environment variables.


=head1 DEPENDENCIES

=over

=item version

=item Perl6::Export::Attrs


=back

=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-power-utils-resource@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Alex Efros  <powerman@asdfgroup.com>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2005,2006 Alex Efros <powerman@asdfgroup.com>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
